@@ -8,7 +8,7 @@
 //!
 //! A command-line tool for enforcing YAML|JSON|TOML|text file contents.
 //!
-//! Use as follows:
+//! # Usage
 //! ```yaml
 //! # conform.yaml
 //! config:
@@ -22,9 +22,26 @@
 //! - file: src/lib.rs
 //!   matches-regex: '(?m)^use' # Ensure this regex is matched in the file
 //!
+//! include:                    # (Recursively) merge config from these urls
+//! - https://example.com/another-conform.yaml
+//!
 //! ```
 //!
 //! ```console
+//! $ you-must-conform --help
+//! you-must-conform 1.1.0
+//! A command-line tool for enforcing YAML|JSON|TOML|text file contents.
+//!
+//! USAGE:
+//!     you-must-conform [OPTIONS] <--file <FILE>|--url <URL>>
+//!
+//! OPTIONS:
+//!     -c, --context <CONTEXT>    The folder to check against the config file [default: .]
+//!     -f, --file <FILE>          The config file to check [default: conform.yaml]
+//!     -h, --help                 Print help information
+//!     -u, --url <URL>            A url to fetch the config file from instead
+//!     -V, --version              Print version information
+//!
 //! $ you-must-conform
 //! Schema not matched in ./Cargo.toml:
 //!     "package" is a required property
@@ -40,7 +57,7 @@ use regex::Regex;
 use serde::{Deserialize, Serialize};
 use std::{
     borrow::Cow,
-    fs,
+    fs, io,
     path::{Path, PathBuf},
 };
 mod json;
@@ -97,67 +114,74 @@ pub fn check_items(
             CheckItem::File { file, check } => {
                 let path = root.join(file);
                 match check {
-                    // This could be buggy - filesystem error will return false
                     FileCheck::Exists {
                         exists: should_exist,
-                    } => match path.is_file() {
-                        true if !should_exist => problems.push(DisallowedFile(path)),
-                        false if should_exist => problems.push(FileNotPresent(path)),
+                    } => match path.metadata() {
+                        Ok(meta) if meta.is_file() && !should_exist => {
+                            problems.push(DisallowedFile(path))
+                        }
+                        Err(err) if err.kind() == io::ErrorKind::NotFound && should_exist => {
+                            problems.push(FileNotPresent(path))
+                        }
                         _ => (),
                     },
                     FileCheck::LooksLike {
                         format,
                         schema: like,
-                    } => {
-                        // TODO check exist, handle appropriately
-                        // Read to string since `toml` doesn't have a from_reader
-                        let s = fs::read_to_string(&path)
-                            .context(format!("Couldn't read {}", path.display()))?;
-                        let d = match format {
-                            FileFormat::Json => {
-                                serde_json::from_str(&s).map_err(anyhow::Error::new)
-                            }
-                            FileFormat::Toml => toml::from_str(&s).map_err(anyhow::Error::new),
-                            FileFormat::Yaml => {
-                                serde_yaml::from_str(&s).map_err(anyhow::Error::new)
-                            }
-                        };
-                        match d {
-                            Ok(v) => {
-                                let schema = JSONSchema::compile(&describe_value(&like))
-                                    .expect("Autogenerated schema generation failed");
+                    } => match path.is_file() {
+                        true => {
+                            // Read to string since `toml` doesn't have a from_reader
+                            let s = fs::read_to_string(&path)
+                                .context(format!("Couldn't read {}", path.display()))?;
+                            let deser_result = match format {
+                                FileFormat::Json => {
+                                    serde_json::from_str(&s).map_err(anyhow::Error::new)
+                                }
+                                FileFormat::Toml => toml::from_str(&s).map_err(anyhow::Error::new),
+                                FileFormat::Yaml => {
+                                    serde_yaml::from_str(&s).map_err(anyhow::Error::new)
+                                }
+                            };
+                            match deser_result {
+                                Ok(v) => {
+                                    let schema = JSONSchema::compile(&describe_value(&like))
+                                        .expect("Autogenerated schema generation failed, please file a bug report.");
 
-                                if let Err(errors) = schema.validate(&v) {
-                                    problems.push(SchemaNotMatched {
-                                        path,
-                                        errors: errors
-                                            .map(|validation_error| ValidationError {
-                                                instance: Cow::Owned(
-                                                    validation_error.instance.into_owned(),
-                                                ),
-                                                ..validation_error
-                                            })
-                                            .collect(),
-                                    })
-                                };
+                                    if let Err(errors) = schema.validate(&v) {
+                                        problems.push(SchemaNotMatched {
+                                            path,
+                                            errors: errors
+                                                .map(|validation_error| ValidationError {
+                                                    instance: Cow::Owned(
+                                                        validation_error.instance.into_owned(),
+                                                    ),
+                                                    ..validation_error
+                                                })
+                                                .collect(),
+                                        })
+                                    };
+                                }
+                                Err(err) => problems.push(InvalidFormat {
+                                    path,
+                                    format: format.into(),
+                                    err,
+                                }),
                             }
-                            Err(err) => problems.push(InvalidFormat {
-                                path,
-                                format: format.into(),
-                                err,
-                            }),
                         }
-                    }
+                        false => problems.push(FileNotPresent(path)),
+                    },
                     FileCheck::MatchesRegex {
                         matches_regex: regex,
-                    } => {
-                        // TODO check exist, handle appropriately
-                        let s = fs::read_to_string(&path)
-                            .context(format!("Couldn't read {}", path.display()))?;
-                        if !regex.is_match(&s) {
-                            problems.push(RegexNotMatched { path, regex })
+                    } => match path.is_file() {
+                        true => {
+                            let s = fs::read_to_string(&path)
+                                .context(format!("Couldn't read {}", path.display()))?;
+                            if !regex.is_match(&s) {
+                                problems.push(RegexNotMatched { path, regex })
+                            }
                         }
-                    }
+                        false => problems.push(FileNotPresent(path)),
+                    },
                 }
             }
         }
